﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.BioBoost.Evaluators;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.BioBoost.Operators.Moves {

  [StorableClass]
  public class MoveEvaluatorAdapter : SingleSuccessorOperator, ISingleObjectiveMoveEvaluator {

    public IConstrainedValueParameter<ISingleObjectiveEvaluator> EvaluatorParameter {
      get { return (IConstrainedValueParameter<ISingleObjectiveEvaluator>) Parameters["Evaluator"]; }
    }

    public ISingleObjectiveEvaluator Evaluator { get { return EvaluatorParameter.Value; } }

    #region construction & cloning

    [StorableConstructor]
    public MoveEvaluatorAdapter(bool isDeserializing) : base(isDeserializing) {}

    public MoveEvaluatorAdapter(MoveEvaluatorAdapter orig, Cloner cloner) : base(orig, cloner) { }

    public MoveEvaluatorAdapter() {
      Parameters.Add(new LookupParameter<DoubleValue>("MoveQuality", "the move quality"));
      var evaluators = new ItemSet<ISingleObjectiveEvaluator>();
      evaluators.Add(new AggregateEvaluator());
      evaluators.Add(new MonolithicEvaluator());
      Parameters.Add(new ConstrainedValueParameter<ISingleObjectiveEvaluator>("Evaluator", "The actual evaluator.", evaluators));
      EvaluatorParameter.Value = EvaluatorParameter.ValidValues.FirstOrDefault(e => e.GetType() == typeof(MonolithicEvaluator));
    }

    public override IDeepCloneable Clone(Cloner cloner) { return new MoveEvaluatorAdapter(this, cloner); }

    #endregion

    public ILookupParameter<DoubleValue> QualityParameter { get { return Evaluator.QualityParameter; } }

    public ILookupParameter<DoubleValue> MoveQualityParameter { get { return (ILookupParameter<DoubleValue>) Parameters["MoveQuality"]; } }

    public override IOperation Apply() {
      var evaluator = (ISingleObjectiveEvaluator)Evaluator.Clone();
      evaluator.QualityParameter.ActualName = MoveQualityParameter.ActualName;
      return new OperationCollection {
        ExecutionContext.CreateChildOperation(evaluator),
        base.Apply(),
      };
    }
  }
}
