﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.BioBoost.Evaluators;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.BioBoost.Operators.Moves {

  [StorableClass]
  public class MoveMakerAdapter : SingleSuccessorOperator, IMoveMaker {

    public IConstrainedValueParameter<ISingleObjectiveEvaluator> EvaluatorParameter { get { return (IConstrainedValueParameter<ISingleObjectiveEvaluator>) Parameters["Evaluator"]; } }
    public ISingleObjectiveEvaluator Evaluator { get { return EvaluatorParameter.Value; } }

    #region construction & cloning
    [StorableConstructor]
    public MoveMakerAdapter(bool isDeserializing) : base(isDeserializing) { }

    public MoveMakerAdapter(MoveMakerAdapter orig, Cloner cloner) : base(orig, cloner) { }

    public MoveMakerAdapter() {
      Parameters.Add(new LookupParameter<DoubleValue>("MoveQuality", "the move quality"));
      Parameters.Add(new ConstrainedValueParameter<ISingleObjectiveEvaluator>("Evaluator", "The actual evaluator.",
        new ItemSet<ISingleObjectiveEvaluator>(ApplicationManager.Manager.GetInstances<ISingleObjectiveEvaluator>())));
      EvaluatorParameter.Value =
        EvaluatorParameter.ValidValues.FirstOrDefault(e => e.GetType() == typeof (MonolithicEvaluator));
    }

    public override IDeepCloneable Clone(Cloner cloner) { return new MoveMakerAdapter(this, cloner); }
    #endregion

    public ILookupParameter<DoubleValue> QualityParameter { get { return Evaluator.QualityParameter; } }

    public ILookupParameter<DoubleValue> MoveQualityParameter { get { return (ILookupParameter<DoubleValue>) Parameters["MoveQuality"]; } }

    public override IOperation Apply() {
      var parentScope = ExecutionContext.Scope.Parent;
      foreach (var variable in ExecutionContext.Scope.Variables) {
        parentScope.Variables.Remove(variable.Name);
        parentScope.Variables.Add(new Variable(variable.Name, variable.Value));
      }
      return base.Apply();
    }
  }
}
